@@ -5,16 +5,25 @@ from types import MethodType
 import llvm.core as lc
 from llvm.core import Type, Constant
 import numpy
+
+import numba
 from numba import types, utils, cgutils, typing, numpy_support, errcode
 from numba.pythonapi import PythonAPI
 from numba.targets.imputils import (user_function, python_attr_impl,
                                     builtin_registry, impl_attribute,
-                                    struct_registry)
-from numba.targets import builtins, iterators, rangeobj
+                                    struct_registry, type_registry)
+from . import arrayobj, builtins, iterators, rangeobj, optional
+try:
+    from . import npdatetime
+except NotImplementedError:
+    pass
 
 
+GENERIC_POINTER = Type.pointer(Type.int(8))
+PYOBJECT = GENERIC_POINTER
+
 LTYPEMAP = {
-    types.pyobject: Type.pointer(Type.int(8)),
+    types.pyobject: PYOBJECT,
 
     types.boolean: Type.int(8),
 
@@ -103,6 +112,9 @@ class BaseContext(object):
     Only POD structure can life across function boundaries by copying the
     data.
     """
+    # True if the target requires strict alignment
+    # Causes exception to be raised if the record members are not aligned.
+    strict_alignment = False
 
     # Use default mangler (no specific requirement)
     mangler = None
@@ -112,11 +124,11 @@ class BaseContext(object):
     implement_pow_as_math_call = False
 
     def __init__(self, typing_context):
-        self.address_size = tuple.__itemsize__ * 8
+        self.address_size = utils.MACHINE_BITS
         self.typing_context = typing_context
 
         self.defns = defaultdict(Overloads)
-        self.attrs = utils.UniqueDict()
+        self.attrs = defaultdict(Overloads)
         self.users = utils.UniqueDict()
 
         self.insert_func_defn(builtin_registry.functions)
@@ -139,15 +151,20 @@ class BaseContext(object):
         obj.metadata = utils.UniqueDict()
         obj.linking = set()
         obj.exceptions = {}
+        obj.cached_internal_func = {}
         return obj
 
+    def link_dependencies(self, module, depends):
+        for lib in depends:
+            module.link_in(lib, preserve=True)
+
     def insert_func_defn(self, defns):
         for defn in defns:
             self.defns[defn.key].append(defn)
 
     def insert_attr_defn(self, defns):
-        for attr in defns:
-            self.attrs[attr.key] = attr
+        for imp in defns:
+            self.attrs[imp.attr].append(imp)
 
     def insert_user_function(self, func, fndesc, libs=()):
         imp = user_function(func, fndesc, libs)
@@ -160,15 +177,27 @@ class BaseContext(object):
 
     def insert_class(self, cls, attrs):
         clsty = types.Object(cls)
-        for name, vtype in utils.dict_iteritems(attrs):
+        for name, vtype in utils.iteritems(attrs):
             imp = python_attr_impl(clsty, name, vtype)
-            self.attrs[imp.key] = imp
+            self.attrs[imp.attr].append(imp)
+
+    def remove_user_function(self, func):
+        """
+        Remove user function *func*.
+        KeyError is raised if the function isn't known to us.
+        """
+        del self.users[func]
+        del self.defns[func]
 
     def get_user_function(self, func):
         return self.users[func]
 
     def get_function_type(self, fndesc):
         """
+        Get the implemented Function type for the high-level *fndesc*.
+        Some parameters can be added or shuffled around.
+        This is kept in sync with call_function() and get_arguments().
+
         Calling Convention
         ------------------
         Returns: -2 for return none in native function;
@@ -176,8 +205,8 @@ class BaseContext(object):
                   0 for success;
                  >0 for user error code.
         Return value is passed by reference as the first argument.
-        It MUST NOT be used if the function is in nopython mode.
-        Actual arguments starts at the 2nd argument position.
+
+        Actual arguments starts at the 2rd argument position.
         Caller is responsible to allocate space for return value.
         """
         argtypes = [self.get_argument_type(aty)
@@ -212,20 +241,21 @@ class BaseContext(object):
         return fn
 
     def insert_const_string(self, mod, string):
-        stringtype = Type.pointer(Type.int(8))
+        stringtype = GENERIC_POINTER
         text = Constant.stringz(string)
         name = ".const.%s" % string
         for gv in mod.global_variables:
             if gv.name == name and gv.type.pointee == text.type:
                 break
         else:
-            gv = mod.add_global_variable(text.type, name=name)
-            gv.global_constant = True
-            gv.initializer = text
-            gv.linkage = lc.LINKAGE_INTERNAL
+            gv = cgutils.global_constant(mod, name, text)
         return Constant.bitcast(gv, stringtype)
 
     def get_arguments(self, func):
+        """
+        Get the Python-level arguments of LLVM *func*.
+        See get_function_type() for the calling convention.
+        """
         return func.args[1:]
 
     def get_argument_type(self, ty):
@@ -237,7 +267,9 @@ class BaseContext(object):
             return self.get_value_type(ty)
 
     def get_return_type(self, ty):
-        if self.is_struct_type(ty):
+        if isinstance(ty, types.Optional):
+            return self.get_return_type(ty.type)
+        elif self.is_struct_type(ty):
             return self.get_argument_type(ty)
         else:
             argty = self.get_argument_type(ty)
@@ -245,24 +277,32 @@ class BaseContext(object):
 
     def get_data_type(self, ty):
         """
-        Get a data representation of the type
+        Get a data representation of the type that is safe for storage.
+        Record data are stored as byte array.
 
         Returns None if it is an opaque pointer
         """
+        try:
+            fac = type_registry.match(ty)
+        except KeyError:
+            pass
+        else:
+            return fac(self, ty)
+
         if (isinstance(ty, types.Dummy) or
                 isinstance(ty, types.Module) or
                 isinstance(ty, types.Function) or
                 isinstance(ty, types.Dispatcher) or
                 isinstance(ty, types.Object) or
                 isinstance(ty, types.Macro)):
-            return Type.pointer(Type.int(8))
+            return PYOBJECT
 
         elif isinstance(ty, types.CPointer):
             dty = self.get_data_type(ty.dtype)
             return Type.pointer(dty)
 
         elif isinstance(ty, types.Optional):
-            return self.get_data_type(ty.type)
+            return self.get_struct_type(self.make_optional(ty))
 
         elif isinstance(ty, types.Array):
             return self.get_struct_type(self.make_array(ty))
@@ -427,6 +467,8 @@ class BaseContext(object):
     def get_setattr(self, attr, sig):
         typ = sig.args[0]
         if isinstance(typ, types.Record):
+            self.sentry_record_alignment(typ, attr)
+
             offset = typ.offset(attr)
             elemty = typ.typeof(attr)
 
@@ -435,17 +477,25 @@ class BaseContext(object):
                 [target, val] = args
                 dptr = cgutils.get_record_member(builder, target, offset,
                                                  self.get_data_type(elemty))
-                self.pack_value(builder, valty, val, dptr)
+                val = context.cast(builder, val, valty, elemty)
+                self.pack_value(builder, elemty, val, dptr)
 
             return _wrap_impl(imp, self, sig)
 
     def get_function(self, fn, sig):
         if isinstance(fn, types.Function):
             key = fn.template.key
+
             if isinstance(key, MethodType):
                 overloads = self.defns[key.im_func]
+
+            elif sig.recvr:
+                sig = typing.signature(sig.return_type,
+                                       *((sig.recvr,) + sig.args))
+                overloads = self.defns[key]
             else:
                 overloads = self.defns[key]
+
         elif isinstance(fn, types.Dispatcher):
             key = fn.overloaded.get_overload(sig.args)
             overloads = self.defns[key]
@@ -457,8 +507,13 @@ class BaseContext(object):
         except NotImplementedError:
             raise Exception("No definition for lowering %s%s" % (key, sig))
 
+    def get_bound_function(self, builder, obj, ty):
+        return obj
+
     def get_attribute(self, val, typ, attr):
         if isinstance(typ, types.Record):
+            # Implement get attribute for records
+            self.sentry_record_alignment(typ, attr)
             offset = typ.offset(attr)
             elemty = typ.typeof(attr)
 
@@ -469,20 +524,39 @@ class BaseContext(object):
                 return self.unpack_value(builder, elemty, dptr)
             return imp
 
-        key = typ, attr
-        try:
-            return self.attrs[key]
-        except KeyError:
-            if isinstance(typ, types.Module):
-                return
-            elif typ.is_parametric:
-                key = type(typ), attr
-                if key in self.attrs:
-                    return self.attrs[key]
+        if isinstance(typ, types.Module):
+            # Implement getattr for module-level globals.
+            # We are treating them as constants.
+            # XXX We shouldn't have to retype this
+            attrty = self.typing_context.resolve_module_constants(typ, attr)
+            if attrty is not None:
+                try:
+                    pyval = getattr(typ.pymod, attr)
+                    llval = self.get_constant(attrty, pyval)
+                except NotImplementedError:
+                    # Module attribute is not a simple constant
+                    # (e.g. it's a function), it will be handled later on.
+                    pass
                 else:
-                    key = type(typ), None
-                    return self.attrs[key]
-            raise
+                    @impl_attribute(typ, attr, attrty)
+                    def imp(context, builder, typ, val):
+                        return llval
+                    return imp
+            # No implementation
+            return None
+
+        # Lookup specific attribute implementation for this type
+        overloads = self.attrs[attr]
+        try:
+            return overloads.find(typing.signature(types.Any, typ))
+        except NotImplementedError:
+            pass
+        # Lookup generic getattr implementation for this type
+        overloads = self.attrs[None]
+        try:
+            return overloads.find(typing.signature(types.Any, typ))
+        except NotImplementedError:
+            raise Exception("No definition for lowering %s.%s" % (typ, attr))
 
     def get_argument_value(self, builder, ty, val):
         """
@@ -502,10 +576,18 @@ class BaseContext(object):
         if ty is types.boolean:
             r = self.get_return_type(ty).pointee
             return builder.zext(val, r)
-
         else:
             return val
 
+    def get_struct_member_value(self, builder, ty, val):
+        """
+        Local value representation to struct member representation
+        """
+        # Currently same as get_return_value.
+        # TODO: make a "TypeLayout" class that describe the layout at
+        #       different situations for a value of a type.
+        return self.get_return_value(builder, ty, val)
+
     def get_value_as_argument(self, builder, ty, val):
         """Prepare local value representation as argument type representation
         """
@@ -525,6 +607,30 @@ class BaseContext(object):
 
         raise NotImplementedError("value %s -> arg %s" % (val.type, argty))
 
+    def return_optional_value(self, builder, retty, valty, value):
+        if valty == types.none:
+            self.return_native_none(builder)
+
+        elif retty == valty:
+            optcls = self.make_optional(retty)
+            optval = optcls(self, builder, value=value)
+
+            validbit = builder.trunc(optval.valid, lc.Type.int(1))
+            with cgutils.ifthen(builder, validbit):
+                self.return_value(builder, optval.data)
+
+            self.return_native_none(builder)
+
+        elif not isinstance(valty, types.Optional):
+            if valty != retty.type:
+                value = self.cast(builder, value, fromty=valty,
+                                  toty=retty.type)
+            self.return_value(builder, value)
+
+        else:
+            raise NotImplementedError("returning {0} for {1}".format(valty,
+                                                                     retty))
+
     def return_value(self, builder, retval):
         fn = cgutils.get_function(builder)
         retptr = fn.args[0]
@@ -682,8 +788,40 @@ class BaseContext(object):
             # then promote to number
             return self.cast(builder, asint, types.int32, toty)
 
+        elif fromty == types.none and isinstance(toty, types.Optional):
+            return self.make_optional_none(builder, toty.type)
+
+        elif isinstance(toty, types.Optional):
+            casted = self.cast(builder, val, fromty, toty.type)
+            return self.make_optional_value(builder, toty.type, casted)
+
+        elif isinstance(fromty, types.Optional):
+            optty = self.make_optional(fromty)
+            optval = optty(self, builder, value=val)
+            validbit = cgutils.as_bool_bit(builder, optval.valid)
+            with cgutils.if_unlikely(builder, builder.not_(validbit)):
+                self.return_errcode(builder, errcode.NONE_TYPE_ERROR)
+
+            return optval.data
+
         raise NotImplementedError("cast", val, fromty, toty)
 
+    def make_optional(self, optionaltype):
+        return optional.make_optional(optionaltype.type)
+
+    def make_optional_none(self, builder, valtype):
+        optcls = optional.make_optional(valtype)
+        optval = optcls(self, builder)
+        optval.valid = cgutils.false_bit
+        return optval._getvalue()
+
+    def make_optional_value(self, builder, valtype, value):
+        optcls = optional.make_optional(valtype)
+        optval = optcls(self, builder)
+        optval.valid = cgutils.true_bit
+        optval.data = value
+        return optval._getvalue()
+
     def is_true(self, builder, typ, val):
         if typ in types.integer_domain:
             return builder.icmp(lc.ICMP_NE, val, Constant.null(val.type))
@@ -697,9 +835,16 @@ class BaseContext(object):
             return builder.or_(real_istrue, imag_istrue)
         raise NotImplementedError("is_true", val, typ)
 
-    def call_function(self, builder, callee, resty, argtys, args):
+    def call_function(self, builder, callee, resty, argtys, args, env=None):
+        """
+        Call the Numba-compiled *callee*, using the same calling
+        convention as in get_function_type().
+        """
+        assert env is None
         retty = callee.args[0].type.pointee
         retval = cgutils.alloca_once(builder, retty)
+        # initialize return value
+        builder.store(lc.Constant.null(retty), retval)
         args = [self.get_value_as_argument(builder, ty, arg)
                 for ty, arg in zip(argtys, args)]
         realargs = [retval] + list(args)
@@ -756,7 +901,7 @@ class BaseContext(object):
 
     def print_string(self, builder, text):
         mod = builder.basic_block.function.module
-        cstring = Type.pointer(Type.int(8))
+        cstring = GENERIC_POINTER
         fnty = Type.function(Type.int(), [cstring])
         puts = mod.get_or_insert_function(fnty, "puts")
         return builder.call(puts, [text])
@@ -766,27 +911,103 @@ class BaseContext(object):
         cstr = self.insert_const_string(mod, str(text))
         self.print_string(builder, cstr)
 
+    def get_struct_member_type(self, member_type):
+        """
+        Get the LLVM type for struct member of type *member_type*.
+        """
+        # get_struct_type() will:
+        # - represent Records as pointers
+        # - represent everything else as plain data
+        if isinstance(member_type, types.Record):
+            return self.get_value_type(member_type)
+        else:
+            return self.get_data_type(member_type)
+
     def get_struct_type(self, struct):
-        fields = [self.get_data_type(v) for _, v in struct._fields]
+        """
+        Get the LLVM struct type for the given Structure class *struct*.
+        """
+        fields = [self.get_struct_member_type(v) for _, v in struct._fields]
         return Type.struct(fields)
 
     def get_dummy_value(self):
         return Constant.null(self.get_dummy_type())
 
     def get_dummy_type(self):
-        return Type.pointer(Type.int(8))
+        return GENERIC_POINTER
+
+    def compile_internal(self, builder, impl, sig, args, locals={},
+                         cache_key=None):
+        """Invoke compiler to implement a function for a nopython function
+
+        Args
+        ----
+        cache_key : hashable
+            A hashable object to use as the key for caching.
+            If it is `None`, no caching is performed.
+        """
+        if cache_key is not None:
+            # Caching is enabled
+            fndesc = self.cached_internal_func.get(cache_key)
+        else:
+            # Caching is disabled
+            fndesc = None
+
+        if fndesc is None:
+            # Compile
+            cres = numba.compiler.compile_internal(self.typing_context, self,
+                                                   impl, sig.args,
+                                                   sig.return_type,
+                                                   locals=locals)
+            llvm_func = cres.llvm_func
+            # Set to linkonce one-definition-rule so that the function
+            # is removed once it is linked.
+            llvm_func.linkage = lc.LINKAGE_LINKONCE_ODR
+            self.add_libs([cres.llvm_module])
+            fndesc = cres.fndesc
+
+            # Do cache if caching is enabled
+            if cache_key is not None:
+                self.cached_internal_func[cache_key] = fndesc
+
+        # Add call to the generated function
+        llvm_mod = cgutils.get_module(builder)
+        fn = self.declare_function(llvm_mod, fndesc)
+        status, res = self.call_function(builder, fn, sig.return_type,
+                                         sig.args, args)
+
+        return res
 
     def optimize(self, module):
         pass
 
+    def finalize(self, func, fndesc):
+        """Perform any necessary work to complete the compilation.
+        An implementation of get_executable() should call finalize().
+        """
+        raise NotImplementedError
+
     def get_executable(self, func, fndesc):
         raise NotImplementedError
 
     def get_python_api(self, builder):
         return PythonAPI(self, builder)
 
+    def sentry_record_alignment(self, rectyp, attr):
+        """
+        Assumes offset starts from a properly aligned location
+        """
+        if self.strict_alignment:
+            offset = rectyp.offset(attr)
+            elemty = rectyp.typeof(attr)
+            align = self.get_abi_sizeof(self.get_data_type(elemty))
+            if offset % align:
+                msg = "{rec}.{attr} of type {type} is not aligned".format(
+                    rec=rectyp, attr=attr, type=elemty)
+                raise TypeError(msg)
+
     def make_array(self, typ):
-        return builtins.make_array(typ)
+        return arrayobj.make_array(typ)
 
     def make_complex(self, typ):
         cls, _ = builtins.get_complex_info(typ)
@@ -814,14 +1035,7 @@ class BaseContext(object):
 
         lldtype = values[0].type
         consts = Constant.array(lldtype, values)
-
-        module = cgutils.get_module(builder)
-
-        data = module.add_global_variable(consts.type, name=".const.array"
-                                                            ".data")
-        data.linkage = lc.LINKAGE_INTERNAL
-        data.global_constant = True
-        data.initializer = consts
+        data = cgutils.global_constant(builder, ".const.array.data", consts)
 
         # Handle shape
         llintp = self.get_value_type(types.intp)
