@@ -1,32 +1,16 @@
 from __future__ import print_function, absolute_import, division
+
+
 import math
 from functools import reduce
 
 from llvm.core import Type, Constant
 import llvm.core as lc
 
-from numba import errcode
-from numba import types, typing, cgutils, utils
-from numba.targets.imputils import (builtin, builtin_attr, implement,
-                                    impl_attribute, impl_attribute_generic,
-                                    iterator_impl, iternext_impl,
-                                    struct_factory)
-from numba.typing import signature
-
-#-------------------------------------------------------------------------------
-
-
-def make_array(ty):
-    dtype = ty.dtype
-    nd = ty.ndim
-
-    class ArrayTemplate(cgutils.Structure):
-        _fields = [('data', types.CPointer(dtype)),
-                   ('shape', types.UniTuple(types.intp, nd)),
-                   ('strides', types.UniTuple(types.intp, nd)),
-                   ('parent', types.pyobject), ]
-
-    return ArrayTemplate
+from .imputils import (builtin, builtin_attr, implement, impl_attribute,
+                       iternext_impl, struct_factory)
+from . import optional
+from .. import typing, types, cgutils, utils, errcode, intrinsics
 
 #-------------------------------------------------------------------------------
 
@@ -136,16 +120,8 @@ def int_utruediv_impl(context, builder, sig, args):
     return builder.fdiv(fx, fy)
 
 
-def int_sfloordiv_impl(context, builder, sig, args):
-    x, y = args
-    cgutils.guard_zero(context, builder, y)
-    return builder.sdiv(x, y)
-
-
-def int_ufloordiv_impl(context, builder, sig, args):
-    x, y = args
-    cgutils.guard_zero(context, builder, y)
-    return builder.udiv(x, y)
+int_sfloordiv_impl = int_sdiv_impl
+int_ufloordiv_impl = int_udiv_impl
 
 
 def int_srem_impl(context, builder, sig, args):
@@ -163,18 +139,23 @@ def int_urem_impl(context, builder, sig, args):
 def int_spower_impl(context, builder, sig, args):
     module = cgutils.get_module(builder)
     x, y = args
-    if y.type.width > 32:
-        y = builder.trunc(y, Type.int(32))
-    elif y.type.width < 32:
-        y = builder.sext(y, Type.int(32))
+
+    # Cast x to float64 to ensure enough precision for the result
+    x = context.cast(builder, x, sig.args[0], types.float64)
+    # Cast y to int32
+    y = context.cast(builder, y, sig.args[1], types.int32)
 
     if context.implement_powi_as_math_call:
-        undersig = typing.signature(sig.return_type, sig.args[0], types.int32)
+        undersig = typing.signature(sig.return_type, types.float64,
+                                    types.int32)
         impl = context.get_function(math.pow, undersig)
-        return impl(builder, (x, y))
+        res = impl(builder, (x, y))
     else:
         powerfn = lc.Function.intrinsic(module, lc.INTR_POWI, [x.type])
-        return builder.call(powerfn, (x, y))
+        res = builder.call(powerfn, (x, y))
+
+    # Cast result back
+    return context.cast(builder, res, types.float64, sig.return_type)
 
 
 def int_upower_impl(context, builder, sig, args):
@@ -195,8 +176,8 @@ def int_upower_impl(context, builder, sig, args):
 
 
 def int_power_func_body(context, builder, x, y):
-    pcounter = builder.alloca(y.type)
-    presult = builder.alloca(x.type)
+    pcounter = cgutils.alloca_once(builder, y.type)
+    presult = cgutils.alloca_once(builder, x.type)
     result = Constant.int(x.type, 1)
     counter = y
     builder.store(counter, pcounter)
@@ -352,6 +333,12 @@ def int_invert_impl(context, builder, sig, args):
     return builder.xor(val, Constant.all_ones(val.type))
 
 
+def bool_invert_impl(context, builder, sig, args):
+    [typ] = sig.args
+    [val] = args
+    return builder.sub(Constant.int(val.type, 1), val)
+
+
 def int_sign_impl(context, builder, sig, args):
     [x] = args
     POS = Constant.int(x.type, 1)
@@ -361,7 +348,7 @@ def int_sign_impl(context, builder, sig, args):
     cmp_zero = builder.icmp(lc.ICMP_EQ, x, ZERO)
     cmp_pos = builder.icmp(lc.ICMP_SGT, x, ZERO)
 
-    presult = builder.alloca(x.type)
+    presult = cgutils.alloca_once(builder, x.type)
 
     bb_zero = cgutils.append_basic_block(builder, ".zero")
     bb_postest = cgutils.append_basic_block(builder, ".postest")
@@ -396,7 +383,7 @@ builtin(implement('<', types.boolean, types.boolean)(int_ult_impl))
 builtin(implement('<=', types.boolean, types.boolean)(int_ule_impl))
 builtin(implement('>', types.boolean, types.boolean)(int_ugt_impl))
 builtin(implement('>=', types.boolean, types.boolean)(int_uge_impl))
-builtin(implement('~', types.boolean)(int_invert_impl))
+builtin(implement('~', types.boolean)(bool_invert_impl))
 
 for ty in types.integer_domain:
     builtin(implement('+', ty, ty)(int_add_impl))
@@ -427,6 +414,7 @@ for ty in types.unsigned_domain:
     builtin(implement('>', ty, ty)(int_ugt_impl))
     builtin(implement('>=', ty, ty)(int_uge_impl))
     builtin(implement('**', types.float64, ty)(int_upower_impl))
+    builtin(implement(pow, types.float64, ty)(int_upower_impl))
     # logical shift for unsigned
     builtin(implement('>>', ty, types.uint32)(int_lshr_impl))
     builtin(implement(types.abs_type, ty)(uint_abs_impl))
@@ -442,9 +430,67 @@ for ty in types.signed_domain:
     builtin(implement('>=', ty, ty)(int_sge_impl))
     builtin(implement(types.abs_type, ty)(int_abs_impl))
     builtin(implement('**', types.float64, ty)(int_spower_impl))
+    builtin(implement(pow, types.float64, ty)(int_spower_impl))
     # arithmetic shift for signed
     builtin(implement('>>', ty, types.uint32)(int_ashr_impl))
 
+builtin(implement('**', types.intp, types.intp)(int_spower_impl))
+builtin(implement(pow, types.intp, types.intp)(int_spower_impl))
+
+def optional_is_none(context, builder, sig, args):
+    """Check if an Optional value is invalid
+    """
+    [lty, rty] = sig.args
+    [lval, rval] = args
+
+    # Make sure None is on the right
+    if lty == types.none:
+        lty, rty = rty, lty
+        lval, rval = rval, lval
+
+    opt_type = lty
+    opt_val = lval
+
+    del lty, rty, lval, rval
+
+    opt = context.make_optional(opt_type)(context, builder, opt_val)
+    return builder.not_(cgutils.as_bool_bit(builder, opt.valid))
+
+
+def optional_is_not_none(context, builder, sig, args):
+    """Check if an Optional value is valid
+    """
+    return builder.not_(optional_is_none(context, builder, sig, args))
+
+
+# None is/not None
+builtin(implement('is', types.none, types.none)(
+    optional.always_return_true_impl))
+
+builtin(implement('is not', types.none, types.none)(
+    optional.always_return_false_impl))
+
+# Optional is None
+builtin(implement('is',
+                  types.Kind(types.Optional), types.none)(optional_is_none))
+
+builtin(implement('is',
+                  types.none, types.Kind(types.Optional))(optional_is_none))
+
+# Optional is not None
+builtin(implement('is not',
+                  types.Kind(types.Optional), types.none)(optional_is_not_none))
+
+builtin(implement('is not',
+                  types.none, types.Kind(types.Optional))(optional_is_not_none))
+
+# Any is/not None
+builtin(implement('is',
+                  types.Any, types.Any)(optional.always_return_false_impl))
+
+builtin(implement('is not',
+                  types.Any, types.Any)(optional.always_return_true_impl))
+
 
 def real_add_impl(context, builder, sig, args):
     return builder.fadd(*args)
@@ -531,9 +577,9 @@ def real_divmod_func_body(context, builder, vx, wx):
     #     }
     #     return Py_BuildValue("(dd)", floordiv, mod);
     # }
-    pmod = builder.alloca(vx.type)
-    pdiv = builder.alloca(vx.type)
-    pfloordiv = builder.alloca(vx.type)
+    pmod = cgutils.alloca_once(builder, vx.type)
+    pdiv = cgutils.alloca_once(builder, vx.type)
+    pfloordiv = cgutils.alloca_once(builder, vx.type)
 
     mod = builder.frem(vx, wx)
     div = builder.fdiv(builder.fsub(vx, mod), wx)
@@ -650,14 +696,8 @@ def real_abs_impl(context, builder, sig, args):
     return impl(builder, args)
 
 def real_negate_impl(context, builder, sig, args):
-    [typ] = sig.args
-    [val] = args
-    val = context.cast(builder, val, typ, sig.return_type)
-    if sig.return_type in types.real_domain:
-        return builder.fsub(context.get_constant(sig.return_type, 0), val)
-    else:
-        return builder.neg(val)
-
+    from . import mathimpl
+    return mathimpl.negate_real(builder, args[0])
 
 def real_positive_impl(context, builder, sig, args):
     [typ] = sig.args
@@ -671,35 +711,23 @@ def real_sign_impl(context, builder, sig, args):
     NEG = Constant.real(x.type, -1)
     ZERO = Constant.real(x.type, 0)
 
-    cmp_zero = builder.fcmp(lc.FCMP_OEQ, x, ZERO)
-    cmp_pos = builder.fcmp(lc.FCMP_OGT, x, ZERO)
+    presult = cgutils.alloca_once(builder, x.type)
 
-    presult = builder.alloca(x.type)
+    is_pos = builder.fcmp(lc.FCMP_OGT, x, ZERO)
+    is_neg = builder.fcmp(lc.FCMP_OLT, x, ZERO)
 
-    bb_zero = cgutils.append_basic_block(builder, ".zero")
-    bb_postest = cgutils.append_basic_block(builder, ".postest")
-    bb_pos = cgutils.append_basic_block(builder, ".pos")
-    bb_neg = cgutils.append_basic_block(builder, ".neg")
-    bb_exit = cgutils.append_basic_block(builder, ".exit")
+    with cgutils.ifelse(builder, is_pos) as (gt_zero, not_gt_zero):
+        with gt_zero:
+            builder.store(POS, presult)
+        with not_gt_zero:
+            with cgutils.ifelse(builder, is_neg) as (lt_zero, not_lt_zero):
+                with lt_zero:
+                    builder.store(NEG, presult)
+                with not_lt_zero:
+                    # For both NaN and 0, the result of sign() is simply
+                    # the input value.
+                    builder.store(x, presult)
 
-    builder.cbranch(cmp_zero, bb_zero, bb_postest)
-
-    with cgutils.goto_block(builder, bb_zero):
-        builder.store(ZERO, presult)
-        builder.branch(bb_exit)
-
-    with cgutils.goto_block(builder, bb_postest):
-        builder.cbranch(cmp_pos, bb_pos, bb_neg)
-
-    with cgutils.goto_block(builder, bb_pos):
-        builder.store(POS, presult)
-        builder.branch(bb_exit)
-
-    with cgutils.goto_block(builder, bb_neg):
-        builder.store(NEG, presult)
-        builder.branch(bb_exit)
-
-    builder.position_at_end(bb_exit)
     return builder.load(presult)
 
 
@@ -712,6 +740,7 @@ for ty in types.real_domain:
     builtin(implement('/', ty, ty)(real_div_impl))
     builtin(implement('%', ty, ty)(real_mod_impl))
     builtin(implement('**', ty, ty)(real_power_impl))
+    builtin(implement(pow, ty, ty)(real_power_impl))
 
     builtin(implement('==', ty, ty)(real_eq_impl))
     builtin(implement('!=', ty, ty)(real_ne_impl))
@@ -750,35 +779,46 @@ def get_complex_info(ty):
 
 
 @builtin_attr
-@impl_attribute(types.complex64, "real", types.float32)
-def complex64_real_impl(context, builder, typ, value):
-    cplx = Complex64(context, builder, value=value)
+@impl_attribute(types.Kind(types.Complex), "real")
+def complex_real_impl(context, builder, typ, value):
+    cplx_cls = context.make_complex(typ)
+    cplx = cplx_cls(context, builder, value=value)
     return cplx.real
 
-
 @builtin_attr
-@impl_attribute(types.complex128, "real", types.float64)
-def complex128_real_impl(context, builder, typ, value):
-    cplx = Complex128(context, builder, value=value)
-    return cplx.real
-
-
-@builtin_attr
-@impl_attribute(types.complex64, "imag", types.float32)
-def complex64_imag_impl(context, builder, typ, value):
-    cplx = Complex64(context, builder, value=value)
+@impl_attribute(types.Kind(types.Complex), "imag")
+def complex_imag_impl(context, builder, typ, value):
+    cplx_cls = context.make_complex(typ)
+    cplx = cplx_cls(context, builder, value=value)
     return cplx.imag
 
+@builtin
+@implement("complex.conjugate", types.Kind(types.Complex))
+def complex_conjugate_impl(context, builder, sig, args):
+    from . import mathimpl
+    cplx_cls = context.make_complex(sig.args[0])
+    z = cplx_cls(context, builder, args[0])
+    z.imag = mathimpl.negate_real(builder, z.imag)
+    return z._getvalue()
 
-@builtin_attr
-@impl_attribute(types.complex128, "imag", types.float64)
-def complex128_imag_impl(context, builder, typ, value):
-    cplx = Complex128(context, builder, value=value)
-    return cplx.imag
+def real_real_impl(context, builder, typ, value):
+    return value
+
+def real_imag_impl(context, builder, typ, value):
+    return cgutils.get_null_value(value.type)
+
+def real_conjugate_impl(context, builder, sig, args):
+    return args[0]
+
+for cls in (types.Float, types.Integer):
+    builtin_attr(impl_attribute(types.Kind(cls), "real")(real_real_impl))
+    builtin_attr(impl_attribute(types.Kind(cls), "imag")(real_imag_impl))
+    builtin(implement("complex.conjugate", types.Kind(cls))(real_conjugate_impl))
 
 
 @builtin
 @implement("**", types.complex128, types.complex128)
+@implement(pow, types.complex128, types.complex128)
 def complex128_power_impl(context, builder, sig, args):
     [ca, cb] = args
     a = Complex128(context, builder, value=ca)
@@ -866,53 +906,46 @@ def complex_mul_impl(context, builder, sig, args):
     return z._getvalue()
 
 
+NAN = float('nan')
+
 def complex_div_impl(context, builder, sig, args):
-    """
-    z = c^2 + d^2
-    (a+bi)/(c+di) = (ac + bd) / z, (bc - ad) / z
-    """
-    [cx, cy] = args
-    complexClass = context.make_complex(sig.args[0])
-    x = complexClass(context, builder, value=cx)
-    y = complexClass(context, builder, value=cy)
-    z = complexClass(context, builder)
-    a = x.real
-    b = x.imag
-    c = y.real
-    d = y.imag
+    def complex_div(a, b):
+        # This is CPython's algorithm (in _Py_c_quot()).
+        areal = a.real
+        aimag = a.imag
+        breal = b.real
+        bimag = b.imag
+        if abs(breal) >= abs(bimag):
+            # Divide tops and bottom by b.real
+            if not breal:
+                return complex(NAN, NAN)
+            ratio = bimag / breal
+            denom = breal + bimag * ratio
+            return complex(
+                (areal + aimag * ratio) / denom,
+                (aimag - areal * ratio) / denom)
+        else:
+            # Divide tops and bottom by b.imag
+            if not bimag:
+                return complex(NAN, NAN)
+            ratio = breal / bimag
+            denom = breal * ratio + bimag
+            return complex(
+                (a.real * ratio + a.imag) / denom,
+                (a.imag * ratio - a.real) / denom)
 
-    ac = builder.fmul(a, c)
-    bd = builder.fmul(b, d)
-    ad = builder.fmul(a, d)
-    bc = builder.fmul(b, c)
-
-    cc = builder.fmul(c, c)
-    dd = builder.fmul(d, d)
-    zz = builder.fadd(cc, dd)
-
-    ac_bd = builder.fadd(ac, bd)
-    bc_ad = builder.fsub(bc, ad)
-
-    cgutils.guard_zero(context, builder, zz)
-    z.real = builder.fdiv(ac_bd, zz)
-    z.imag = builder.fdiv(bc_ad, zz)
-    return z._getvalue()
+    return context.compile_internal(builder, complex_div, sig, args)
 
 
 def complex_negate_impl(context, builder, sig, args):
+    from . import mathimpl
     [typ] = sig.args
     [val] = args
     cmplxcls = context.make_complex(typ)
-    cmplx = cmplxcls(context, builder, val)
-
-    real = cmplx.real
-    imag = cmplx.imag
-
-    zero = Constant.real(real.type, 0)
-
+    cmplx = cmplxcls(context, builder, value=val)
     res = cmplxcls(context, builder)
-    res.real = builder.fsub(zero, real)
-    res.imag = builder.fsub(zero, imag)
+    res.real = mathimpl.negate_real(builder, cmplx.real)
+    res.imag = mathimpl.negate_real(builder, cmplx.imag)
     return res._getvalue()
 
 
@@ -947,19 +980,13 @@ def complex_abs_impl(context, builder, sig, args):
     """
     abs(z) := hypot(z.real, z.imag)
     """
-    [typ] = sig.args
-    [val] = args
-    cmplxcls = context.make_complex(typ)
-    flty = typ.underlying_float
-    cmplx = cmplxcls(context, builder, val)
-    [x, y] = cmplx.real, cmplx.imag
-    hypotsig = signature(sig.return_type, flty, flty)
-    hypotimp = context.get_function(math.hypot, hypotsig)
-    return hypotimp(builder, [x, y])
+    def complex_abs(z):
+        return math.hypot(z.real, z.imag)
+
+    return context.compile_internal(builder, complex_abs, sig, args)
 
 
-for ty, cls in zip([types.complex64, types.complex128],
-                   [Complex64, Complex128]):
+for ty in types.complex_domain:
     builtin(implement("+", ty, ty)(complex_add_impl))
     builtin(implement("-", ty, ty)(complex_sub_impl))
     builtin(implement("*", ty, ty)(complex_mul_impl))
@@ -1181,310 +1208,6 @@ def getitem_unituple(context, builder, sig, args):
     builder.position_at_end(bbend)
     return phinode
 
-
-@builtin
-@implement('getitem', types.Kind(types.Array), types.intp)
-def getitem_array1d_intp(context, builder, sig, args):
-    aryty, _ = sig.args
-    if aryty.ndim != 1:
-        # TODO
-        raise NotImplementedError("1D indexing into %dD array" % aryty.ndim)
-
-    ary, idx = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, [idx],
-                                   wraparound=context.metadata['wraparound'])
-    return context.unpack_value(builder, aryty.dtype, ptr)
-
-
-@builtin
-@implement('getitem', types.Kind(types.Array), types.slice3_type)
-def getitem_array1d_slice(context, builder, sig, args):
-    aryty, _ = sig.args
-    if aryty.ndim != 1:
-        # TODO
-        raise NotImplementedError("1D indexing into %dD array" % aryty.ndim)
-
-    ary, idx = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, value=ary)
-
-    shapes = cgutils.unpack_tuple(builder, ary.shape, aryty.ndim)
-
-    slicestruct = Slice(context, builder, value=idx)
-    cgutils.normalize_slice(builder, slicestruct, shapes[0])
-
-    dataptr = cgutils.get_item_pointer(builder, aryty, ary,
-                                       [slicestruct.start],
-                                       wraparound=context.metadata['wraparound'])
-
-    retstty = make_array(sig.return_type)
-    retary = retstty(context, builder)
-
-    shape = cgutils.get_range_from_slice(builder, slicestruct)
-    retary.shape = cgutils.pack_array(builder, [shape])
-
-    stride = cgutils.get_strides_from_slice(builder, aryty.ndim, ary.strides,
-                                            slicestruct, 0)
-    retary.strides = cgutils.pack_array(builder, [stride])
-    retary.data = dataptr
-
-    return retary._getvalue()
-
-
-@builtin
-@implement('getitem', types.Kind(types.Array),
-           types.Kind(types.UniTuple))
-def getitem_array_unituple(context, builder, sig, args):
-    aryty, idxty = sig.args
-    ary, idx = args
-
-    ndim = aryty.ndim
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    if idxty.dtype == types.slice3_type:
-        # Slicing
-        raw_slices = cgutils.unpack_tuple(builder, idx, aryty.ndim)
-        slices = [Slice(context, builder, value=sl) for sl in raw_slices]
-        for sl, sh in zip(slices,
-                          cgutils.unpack_tuple(builder, ary.shape, ndim)):
-            cgutils.normalize_slice(builder, sl, sh)
-        indices = [sl.start for sl in slices]
-        dataptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                           wraparound=context.metadata['wraparound'])
-        # Build array
-        retstty = make_array(sig.return_type)
-        retary = retstty(context, builder)
-        retary.data = dataptr
-        shapes = [cgutils.get_range_from_slice(builder, sl)
-                  for sl in slices]
-        retary.shape = cgutils.pack_array(builder, shapes)
-        strides = [cgutils.get_strides_from_slice(builder, ndim, ary.strides,
-                                                  sl, i)
-                   for i, sl in enumerate(slices)]
-
-        retary.strides = cgutils.pack_array(builder, strides)
-
-        return retary._getvalue()
-    else:
-        # Indexing
-        assert idxty.dtype == types.intp
-        indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
-        indices = [context.cast(builder, i, t, types.intp)
-                   for t, i in zip(idxty, indices)]
-        ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                       wraparound=context.metadata['wraparound'])
-
-        return context.unpack_value(builder, aryty.dtype, ptr)
-
-
-@builtin
-@implement('getitem', types.Kind(types.Array),
-           types.Kind(types.Tuple))
-def getitem_array_tuple(context, builder, sig, args):
-    aryty, idxty = sig.args
-    ary, idx = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    ndim = aryty.ndim
-    if isinstance(sig.return_type, types.Array):
-        # Slicing
-        raw_indices = cgutils.unpack_tuple(builder, idx, aryty.ndim)
-        start = []
-        shapes = []
-        strides = []
-
-        oshapes = cgutils.unpack_tuple(builder, ary.shape, ndim)
-        for ax, (indexval, idxty) in enumerate(zip(raw_indices, idxty)):
-            if idxty == types.slice3_type:
-                slice = Slice(context, builder, value=indexval)
-                cgutils.normalize_slice(builder, slice, oshapes[ax])
-                start.append(slice.start)
-                shapes.append(cgutils.get_range_from_slice(builder, slice))
-                strides.append(cgutils.get_strides_from_slice(builder, ndim,
-                                                              ary.strides,
-                                                              slice, ax))
-            else:
-                ind = context.cast(builder, indexval, idxty, types.intp)
-                start.append(ind)
-
-        dataptr = cgutils.get_item_pointer(builder, aryty, ary, start,
-                                           wraparound=context.metadata['wraparound'])
-        # Build array
-        retstty = make_array(sig.return_type)
-        retary = retstty(context, builder)
-        retary.data = dataptr
-        retary.shape = cgutils.pack_array(builder, shapes)
-        retary.strides = cgutils.pack_array(builder, strides)
-        return retary._getvalue()
-    else:
-        # Indexing
-        indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
-        indices = [context.cast(builder, i, t, types.intp)
-                   for t, i in zip(idxty, indices)]
-        ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                       wraparound=context.metadata['wraparound'])
-
-        return context.unpack_value(builder, aryty.dtype, ptr)
-
-
-@builtin
-@implement('setitem', types.Kind(types.Array), types.intp,
-           types.Any)
-def setitem_array1d(context, builder, sig, args):
-    aryty, _, valty = sig.args
-    ary, idx, val = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, [idx],
-                                   wraparound=context.metadata['wraparound'])
-
-    val = context.cast(builder, val, valty, aryty.dtype)
-
-    context.pack_value(builder, aryty.dtype, val, ptr)
-
-
-@builtin
-@implement('setitem', types.Kind(types.Array),
-           types.Kind(types.UniTuple), types.Any)
-def setitem_array_unituple(context, builder, sig, args):
-    aryty, idxty, valty = sig.args
-    ary, idx, val = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    # TODO: other than layout
-    indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
-    indices = [context.cast(builder, i, t, types.intp)
-               for t, i in zip(idxty, indices)]
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                   wraparound=context.metadata['wraparound'])
-    context.pack_value(builder, aryty.dtype, val, ptr)
-
-
-@builtin
-@implement('setitem', types.Kind(types.Array),
-           types.Kind(types.Tuple), types.Any)
-def setitem_array_tuple(context, builder, sig, args):
-    aryty, idxty, valty = sig.args
-    ary, idx, val = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    # TODO: other than layout
-    indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
-    indices = [context.cast(builder, i, t, types.intp)
-               for t, i in zip(idxty, indices)]
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                   wraparound=context.metadata['wraparound'])
-    context.pack_value(builder, aryty.dtype, val, ptr)
-
-
-@builtin
-@implement('setitem', types.Kind(types.Array),
-           types.Kind(types.Tuple), types.Any)
-def setitem_array_tuple(context, builder, sig, args):
-    aryty, idxty, valty = sig.args
-    ary, idx, val = args
-
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-
-    # TODO: other than layout
-    indices = cgutils.unpack_tuple(builder, idx, count=len(idxty))
-    indices = [context.cast(builder, i, t, types.intp)
-               for t, i in zip(idxty, indices)]
-    ptr = cgutils.get_item_pointer(builder, aryty, ary, indices,
-                                   wraparound=context.metadata['wraparound'])
-
-    context.pack_value(builder, aryty.dtype, val, ptr)
-
-
-@builtin
-@implement(types.len_type, types.Kind(types.Array))
-def array_len(context, builder, sig, args):
-    (aryty,) = sig.args
-    (ary,) = args
-    arystty = make_array(aryty)
-    ary = arystty(context, builder, ary)
-    shapeary = ary.shape
-    return builder.extract_value(shapeary, 0)
-
-
-#-------------------------------------------------------------------------------
-
-
-@builtin_attr
-@impl_attribute(types.Array, "shape", types.Kind(types.UniTuple))
-def array_shape(context, builder, typ, value):
-    arrayty = make_array(typ)
-    array = arrayty(context, builder, value)
-    return array.shape
-
-
-@builtin_attr
-@impl_attribute(types.Array, "strides", types.Kind(types.UniTuple))
-def array_strides(context, builder, typ, value):
-    arrayty = make_array(typ)
-    array = arrayty(context, builder, value)
-    return array.strides
-
-
-@builtin_attr
-@impl_attribute(types.Array, "ndim", types.intp)
-def array_ndim(context, builder, typ, value):
-    return context.get_constant(types.intp, typ.ndim)
-
-
-@builtin_attr
-@impl_attribute(types.Array, "size", types.intp)
-def array_size(context, builder, typ, value):
-    arrayty = make_array(typ)
-    array = arrayty(context, builder, value)
-    dims = cgutils.unpack_tuple(builder, array.shape, typ.ndim)
-    return reduce(builder.mul, dims[1:], dims[0])
-
-
-@builtin_attr
-@impl_attribute_generic(types.Array)
-def array_record_getattr(context, builder, typ, value, attr):
-    arrayty = make_array(typ)
-    array = arrayty(context, builder, value)
-
-    rectype = typ.dtype
-    dtype = rectype.typeof(attr)
-    offset = rectype.offset(attr)
-
-    resty = types.Array(dtype, ndim=typ.ndim, layout='A')
-
-    raryty = make_array(resty)
-
-    rary = raryty(context, builder)
-    rary.shape = array.shape
-
-    constoffset = context.get_constant(types.intp, offset)
-    unpackedstrides = cgutils.unpack_tuple(builder, array.strides, typ.ndim)
-    newstrides = [builder.add(s, constoffset) for s in unpackedstrides]
-
-    rary.strides = array.strides
-
-    llintp = context.get_value_type(types.intp)
-    newdata = builder.add(builder.ptrtoint(array.data, llintp), constoffset)
-    newdataptr = builder.inttoptr(newdata, rary.data.type)
-    rary.data = newdataptr
-
-    return rary._getvalue()
-
 #-------------------------------------------------------------------------------
 
 
@@ -1613,23 +1336,26 @@ def float_impl(context, builder, sig, args):
 @builtin
 @implement(complex, types.VarArg)
 def complex_impl(context, builder, sig, args):
+    complex_type = sig.return_type
+    float_type = complex_type.underlying_float
+    complex_cls = context.make_complex(complex_type)
     if len(sig.args) == 1:
         [argty] = sig.args
         [arg] = args
         if isinstance(argty, types.Complex):
-            # Cast Complex* to Complex128
-            return context.cast(builder, arg, argty, types.complex128)
+            # Cast Complex* to Complex*
+            return context.cast(builder, arg, argty, complex_type)
         else:
-            real = context.cast(builder, arg, argty, types.float64)
-            imag = context.get_constant(types.float64, 0)
+            real = context.cast(builder, arg, argty, float_type)
+            imag = context.get_constant(float_type, 0)
 
     elif len(sig.args) == 2:
         [realty, imagty] = sig.args
         [real, imag] = args
-        real = context.cast(builder, real, realty, types.float64)
-        imag = context.cast(builder, imag, imagty, types.float64)
+        real = context.cast(builder, real, realty, float_type)
+        imag = context.cast(builder, imag, imagty, float_type)
 
-    cmplx = Complex128(context, builder)
+    cmplx = complex_cls(context, builder)
     cmplx.real = real
     cmplx.imag = imag
     return cmplx._getvalue()
@@ -1647,3 +1373,29 @@ def math_pi_impl(context, builder, typ, value):
 def math_e_impl(context, builder, typ, value):
     return context.get_constant(types.float64, math.e)
 
+# -----------------------------------------------------------------------------
+
+@builtin
+@implement(intrinsics.array_ravel, types.Kind(types.Array))
+def array_ravel_impl(context, builder, sig, args):
+    [arrty] = sig.args
+    [arr] = args
+    flatarrty = sig.return_type
+
+    flatarrcls = context.make_array(flatarrty)
+    arrcls = context.make_array(arrty)
+
+    flatarr = flatarrcls(context, builder)
+    arr = arrcls(context, builder, value=arr)
+
+    shapes = cgutils.unpack_tuple(builder, arr.shape, arrty.ndim)
+    size = reduce(builder.mul, shapes)
+    strides = cgutils.unpack_tuple(builder, arr.strides, arrty.ndim)
+    unit_stride = strides[0] if arrty.layout == 'F' else strides[-1]
+
+    flatarr.data = arr.data
+    flatarr.shape = cgutils.pack_array(builder, [size])
+    flatarr.strides = cgutils.pack_array(builder, [unit_stride])
+
+    return flatarr._getvalue()
+
