@@ -1,23 +1,33 @@
 from __future__ import print_function, division, absolute_import
 import sys
-import contextlib
+import functools
+from numba import config
 
-import numba.unittest_support as unittest
 
-from numba.tests import NumbaTestProgram
-from numba.utils import StringIO
+def allow_interpreter_mode(fn):
+    """Temporarily re-enable intepreter mode
+    """
+    @functools.wraps(fn)
+    def _core(*args, **kws):
+        config.COMPATIBILITY_MODE = True
+        try:
+            fn(*args, **kws)
+        finally:
+            config.COMPATIBILITY_MODE = False
+    return _core
 
 
 def discover_tests(startdir):
     """Discover test under a directory
     """
-
+    # Avoid importing unittest
+    from numba import unittest_support as unittest
     loader = unittest.TestLoader()
     suite = loader.discover(startdir)
     return suite
 
 
-def run_tests(suite, xmloutput=None):
+def run_tests(suite, xmloutput=None, verbosity=1):
     """
     args
     ----
@@ -25,15 +35,19 @@ def run_tests(suite, xmloutput=None):
         A suite of all tests to run
     - xmloutput [str or None]
         Path of XML output directory (optional)
+    - verbosity [int]
+        Verbosity level of tests output
 
     Returns the TestResult object after running the test *suite*.
     """
+    from numba.tests import NumbaTestProgram
+
     if xmloutput is not None:
         import xmlrunner
         runner = xmlrunner.XMLTestRunner(output=xmloutput)
     else:
         runner = None
-    prog = NumbaTestProgram(suite=suite, testRunner=runner, exit=False)
+    prog = NumbaTestProgram(suite=suite, testRunner=runner, exit=False, verbosity=verbosity)
     return prog.result
 
 
