@@ -17,9 +17,8 @@ from __future__ import print_function, division, absolute_import
 from pprint import pprint
 import itertools
 
-from numba import ir, types, utils, config, ctypes_utils, cffi_support
+from numba import ir, types, utils, config, six
 from numba.config import PYVERSION
-from numba import numpy_support
 from numba.utils import builtins
 
 RANGE_ITER_OBJECTS = (builtins.range,)
@@ -48,6 +47,11 @@ class TypeVar(object):
         if not types:
             return
 
+        # Sentry for None
+        for ty in types:
+            if ty is None:
+                raise TypeError("Using None as variable type")
+
         nbefore = len(self.typeset)
 
         if self.locked:
@@ -109,7 +113,9 @@ class ConstrainNetwork(object):
             except TypingError:
                 raise
             except Exception as e:
-                raise TypingError("Internal error:\n%s" % e, constrain.loc)
+                msg = "Internal error at {con}:\n{err}"
+                raise TypingError(msg.format(con=constrain, err=e),
+                                  loc=constrain.loc)
 
 
 class Propagate(object):
@@ -224,6 +230,7 @@ class CallConstrain(object):
 
     def resolve(self, context, typevars, fnty):
         assert not self.kws, "Keyword argument is not supported, yet"
+        assert fnty
         argtypes = [typevars[a.name].get() for a in self.args]
         restypes = []
         # Case analysis for each combination of argument types.
@@ -260,9 +267,15 @@ class GetAttrConstrain(object):
                 args = (self.attr, ty, self.value.name, self.inst)
                 msg = "Unknown attribute '%s' for %s %s %s" % args
                 raise TypingError(msg, loc=self.inst.loc)
-            restypes.append(attrty)
+            else:
+                assert attrty
+                restypes.append(attrty)
         typevars[self.target].add_types(*restypes)
 
+    def __repr__(self):
+        return 'resolving type of attribute "{attr}" of "{value}"'.format(
+            value=self.value, attr=self.attr)
+
 
 class SetItemConstrain(object):
     def __init__(self, target, index, value, loc):
@@ -339,7 +352,7 @@ class TypeInferer(object):
 
     def dump(self):
         print('---- type variables ----')
-        pprint(utils.dict_values(self.typevars))
+        pprint(list(six.itervalues(self.typevars)))
 
     def seed_type(self, name, typ):
         """All arguments should be seeded.
@@ -349,13 +362,13 @@ class TypeInferer(object):
     def seed_return(self, typ):
         """Seeding of return value is optional.
         """
-        for blk in utils.dict_itervalues(self.blocks):
+        for blk in utils.itervalues(self.blocks):
             inst = blk.terminator
             if isinstance(inst, ir.Return):
                 self.typevars[inst.value.name].lock(typ)
 
     def build_constrain(self):
-        for blk in utils.dict_itervalues(self.blocks):
+        for blk in utils.itervalues(self.blocks):
             for inst in blk.body:
                 self.constrain_statement(inst)
 
@@ -434,7 +447,7 @@ class TypeInferer(object):
 
     def get_return_type(self, typemap):
         rettypes = set()
-        for blk in utils.dict_itervalues(self.blocks):
+        for blk in utils.itervalues(self.blocks):
             term = blk.terminator
             if isinstance(term, ir.Return):
                 rettypes.add(typemap[term.value.name])
@@ -458,7 +471,7 @@ class TypeInferer(object):
         The sum of all lengths of type sets is a cheap and accurate
         description of our progress.
         """
-        return sum(len(tv) for tv in utils.dict_itervalues(self.typevars))
+        return sum(len(tv) for tv in utils.itervalues(self.typevars))
 
     def constrain_statement(self, inst):
         if isinstance(inst, ir.Assign):
@@ -534,6 +547,10 @@ class TypeInferer(object):
 
     def typeof_global(self, inst, target, gvar):
         typ = self.context.resolve_value_type(gvar.value)
+        if isinstance(typ, types.Array):
+            # Global array in nopython mode is constant
+            typ = typ.copy(layout='C', const=True)
+
         if typ is not None:
             self.sentry_modified_builtin(inst, gvar)
             self.typevars[target.name].lock(typ)
@@ -587,13 +604,14 @@ class TypeInferer(object):
                                          value=expr.value, loc=inst.loc,
                                          inst=inst)
             self.constrains.append(constrain)
-        elif expr.op == 'getitem':
-            self.typeof_intrinsic_call(inst, target, expr.op, expr.target,
-                                       expr.index, loc=inst.loc)
         elif expr.op == 'build_tuple':
             constrain = BuildTupleConstrain(target.name, items=expr.items,
                                             loc=inst.loc)
             self.constrains.append(constrain)
+        elif expr.op == 'cast':
+            self.constrains.append(Propagate(dst=target.name,
+                                             src=expr.value.name,
+                                             loc=inst.loc))
         else:
             raise NotImplementedError(type(expr), expr)
 
