@@ -3,46 +3,99 @@ These type objects do not have a fixed machine representation.  It is up to
 the targets to choose their representation.
 """
 from __future__ import print_function, division, absolute_import
-from collections import defaultdict
 
+import itertools
 import numpy
+import weakref
 
-from . import utils
+from .six import add_metaclass
+from . import npdatetime, utils
 
 
+# Types are added to a global registry (_typecache) in order to assign
+# them unique integer codes for fast matching in _dispatcher.c.
+# However, we also want types to be disposable, therefore we ensure
+# each type is interned as a weak reference, so that it lives only as
+# long as necessary to keep a stable type code.
+_typecodes = itertools.count()
+
 def _autoincr():
-    n = len(_typecache)
+    n = next(_typecodes)
     # 4 billion types should be enough, right?
-    assert n <= 2 ** 32, "Limited to 4billion types"
+    assert n < 2 ** 32, "Limited to 4 billion types"
     return n
 
+_typecache = {}
 
-_typecache = defaultdict(_autoincr)
+def _on_type_disposal(wr, _pop=_typecache.pop):
+    _pop(wr, None)
 
 
+class _TypeMetaclass(type):
+    """
+    A metaclass that will intern instances after they are created.
+    This is done by first creating a new instance (including calling
+    __init__, which sets up the required attributes for equality
+    and hashing), then looking it up in the _typecache registry.
+    """
+
+    def __call__(cls, *args, **kwargs):
+        """
+        Instantiate *cls* (a Type subclass, presumably) and intern it.
+        If an interned instance already exists, it is returned, otherwise
+        the new instance is returned.
+        """
+        inst = type.__call__(cls, *args, **kwargs)
+        # Try to intern the created instance
+        wr = weakref.ref(inst, _on_type_disposal)
+        orig = _typecache.get(wr)
+        orig = orig and orig()
+        if orig is not None:
+            return orig
+        else:
+            inst._code = _autoincr()
+            _typecache[wr] = wr
+            inst.post_init()
+            return inst
+
+
+@add_metaclass(_TypeMetaclass)
 class Type(object):
     """
     The default behavior is to provide equality through `name` attribute.
     Two types are equal if there `name` are equal.
     Subclass can refine this behavior.
     """
-    __slots__ = '_code', 'name', 'is_parametric'
 
     mutable = False
 
     def __init__(self, name, param=False):
         self.name = name
         self.is_parametric = param
-        self._code = _typecache[self]
+
+    def post_init(self):
+        """
+        A method called when the instance is fully initialized and has
+        a registered typecode in its _code attribute.  Does nothing by
+        default, but can be overriden.
+        """
+
+    @property
+    def key(self):
+        """
+        A property used for __eq__, __ne__ and __hash__.  Can be overriden
+        in subclasses.
+        """
+        return self.name
 
     def __repr__(self):
         return self.name
 
     def __hash__(self):
-        return hash(self.name)
+        return hash(self.key)
 
     def __eq__(self, other):
-        return self.__class__ is other.__class__ and self.name == other.name
+        return self.__class__ is other.__class__ and self.key == other.key
 
     def __ne__(self, other):
         return not (self == other)
@@ -150,6 +203,36 @@ class Complex(Type):
             return NotImplemented
         return self.bitwidth < other.bitwidth
 
+
+class _NPDatetimeBase(Type):
+    """
+    Common base class for numpy.datetime64 and numpy.timedelta64.
+    """
+
+    def __init__(self, unit, *args, **kws):
+        name = '%s(%s)' % (self.type_name, unit)
+        self.unit = unit
+        self.unit_code = npdatetime.DATETIME_UNITS[self.unit]
+        super(_NPDatetimeBase, self).__init__(name, *args, **kws)
+
+    def __lt__(self, other):
+        if self.__class__ is not other.__class__:
+            return NotImplemented
+        # A coarser-grained unit is "smaller", i.e. less precise values
+        # can be represented (but the magnitude of representable values is
+        # also greater...).
+        return self.unit_code < other.unit_code
+
+
+@utils.total_ordering
+class NPTimedelta(_NPDatetimeBase):
+    type_name = 'timedelta64'
+
+@utils.total_ordering
+class NPDatetime(_NPDatetimeBase):
+    type_name = 'datetime64'
+
+
 class Prototype(Type):
     def __init__(self, args, return_type):
         self.args = args
@@ -163,6 +246,22 @@ class Dummy(Type):
     For type that does not really have a representation and is compatible
     with a void*.
     """
+    pass
+
+
+class Phantom(Dummy):
+    """
+    A type that cannot be materialized.  A Phantom cannot be used as
+    argument or return type.
+    """
+    pass
+
+
+class Opaque(Dummy):
+    """
+    A type that is a opaque pointer.
+    """
+    pass
 
 
 class Kind(Type):
@@ -170,12 +269,9 @@ class Kind(Type):
         self.of = of
         super(Kind, self).__init__("kind(%s)" % of)
 
-    def __eq__(self, other):
-        if isinstance(other, Kind):
-            return self.of == other.of
-
-    def __hash__(self):
-        return hash(self.of)
+    @property
+    def key(self):
+        return self.of
 
 
 class Module(Type):
@@ -183,12 +279,9 @@ class Module(Type):
         self.pymod = pymod
         super(Module, self).__init__("Module(%s)" % pymod)
 
-    def __eq__(self, other):
-        if isinstance(other, Module):
-            return self.pymod == other.pymod
-
-    def __hash__(self):
-        return hash(self.pymod)
+    @property
+    def key(self):
+        return self.pymod
 
 
 class Macro(Type):
@@ -197,13 +290,9 @@ class Macro(Type):
         cls = type(self)
         super(Macro, self).__init__("%s(%s)" % (cls.__name__, template))
 
-    def __eq__(self, other):
-        if isinstance(other, Macro):
-            return self.template == other.template
-
-    def __hash__(self):
-        # FIXME maybe this should not be hashable
-        return hash(self.template)
+    @property
+    def key(self):
+        return self.template
 
 
 class Function(Type):
@@ -211,31 +300,56 @@ class Function(Type):
         self.template = template
         cls = type(self)
         # TODO template is mutable.  Should use different naming scheme
-        super(Function, self).__init__("%s(%s)" % (cls.__name__, template))
+        super(Function, self).__init__("%s(%s)" % (cls.__name__, template.key))
 
-    def __eq__(self, other):
-        if isinstance(other, Function):
-            return self.template == other.template
-
-    def __hash__(self):
-        # FIXME maybe this should not be hashable
-        return hash(self.template)
+    @property
+    def key(self):
+        return self.template
 
     def extend(self, template):
         self.template.cases.extend(template.cases)
 
 
-class Dispatcher(Type):
-    def __init__(self, overloaded):
-        self.overloaded = overloaded
-        super(Dispatcher, self).__init__("Dispatcher(%s)" % overloaded)
+class WeakType(Type):
+    """
+    Base class for types parametered by a mortal object, to which only
+    a weak reference is kept.
+    """
+
+    def _store_object(self, obj):
+        self._wr = weakref.ref(obj)
+
+    def _get_object(self):
+        obj = self._wr()
+        if obj is None:
+            raise ReferenceError("underlying object has vanished")
+        return obj
+
+    @property
+    def key(self):
+        return self._wr
 
     def __eq__(self, other):
-        if isinstance(other, Dispatcher):
-            return self.overloaded is other.overloaded
+        if type(self) is type(other):
+            obj = self._wr()
+            return obj is not None and obj is other._wr()
 
     def __hash__(self):
-        return hash(self.overloaded)
+        return Type.__hash__(self)
+
+
+class Dispatcher(WeakType):
+
+    def __init__(self, overloaded):
+        self._store_object(overloaded)
+        super(Dispatcher, self).__init__("Dispatcher(%s)" % overloaded)
+
+    @property
+    def overloaded(self):
+        """
+        A strong reference to the underlying Dispatcher instance.
+        """
+        return self._get_object()
 
 
 class FunctionPointer(Function):
@@ -244,6 +358,18 @@ class FunctionPointer(Function):
         super(FunctionPointer, self).__init__(template)
 
 
+class BoundFunction(Function):
+    def __init__(self, template, this):
+        self.this = this
+        newcls = type(template.__name__ + '.' + str(this), (template,),
+                      dict(this=this))
+        super(BoundFunction, self).__init__(newcls)
+
+    @property
+    def key(self):
+        return (self.template.__name__, self.this)
+
+
 class Method(Function):
     def __init__(self, template, this):
         self.this = this
@@ -251,13 +377,9 @@ class Method(Function):
                       dict(this=this))
         super(Method, self).__init__(newcls)
 
-    def __eq__(self, other):
-        if isinstance(other, Method):
-            return (self.template.__name__ == other.template.__name__ and
-                    self.this == other.this)
-
-    def __hash__(self):
-        return hash((self.template.__name__, self.this))
+    @property
+    def key(self):
+        return (self.template.__name__, self.this)
 
 
 class Pair(Type):
@@ -271,13 +393,9 @@ class Pair(Type):
         name = "pair<%s, %s>" % (first_type, second_type)
         super(Pair, self).__init__(name=name)
 
-    def __eq__(self, other):
-        if isinstance(other, Pair):
-            return (self.first_type == other.first_type and
-                    self.second_type == other.second_type)
-
-    def __hash__(self):
-        return hash((self.first_type, self.second_type))
+    @property
+    def key(self):
+        return self.first_type, self.second_type
 
 
 class IterableType(Type):
@@ -293,13 +411,6 @@ class SimpleIterableType(IterableType):
         self.iterator_type = iterator_type
         super(SimpleIterableType, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if other.__class__ is self.__class__:
-            return self.name == other.name
-
-    def __hash__(self):
-        return hash(self.name)
-
 
 class IteratorType(IterableType):
     """
@@ -318,21 +429,28 @@ class SimpleIteratorType(IteratorType):
         self.yield_type = yield_type
         super(SimpleIteratorType, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if other.__class__ is self.__class__:
-            return self.name == other.name
-
-    def __hash__(self):
-        return hash(self.name)
-
 
 class RangeType(SimpleIterableType):
     pass
 
+
 class RangeIteratorType(SimpleIteratorType):
     pass
 
 
+class NumpyFlatType(IteratorType):
+    def __init__(self, arrty):
+        self.array_type = arrty
+        self.yield_type = arrty.dtype
+        name = "array.flat({arrayty})".format(arrayty=arrty)
+        super(NumpyFlatType, self).__init__(name, param=True)
+
+
+    @property
+    def key(self):
+        return self.array_type
+
+
 class EnumerateType(IteratorType):
     """
     Type class for `enumerate` objects.
@@ -345,12 +463,9 @@ class EnumerateType(IteratorType):
         name = 'enumerate(%s)' % (self.source_type)
         super(EnumerateType, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, EnumerateType):
-            return self.source_type == other.source_type
-
-    def __hash__(self):
-        return hash(self.source_type)
+    @property
+    def key(self):
+        return self.source_type
 
 
 class ZipType(IteratorType):
@@ -365,12 +480,9 @@ class ZipType(IteratorType):
         name = 'zip(%s)' % ', '.join(str(tp) for tp in self.source_types)
         super(ZipType, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, ZipType):
-            return self.source_types == other.source_types
-
-    def __hash__(self):
-        return hash(self.source_types)
+    @property
+    def key(self):
+        return self.source_types
 
 
 class CharSeq(Type):
@@ -381,12 +493,9 @@ class CharSeq(Type):
         name = "[char x %d]" % count
         super(CharSeq, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, CharSeq):
-            return self.count == other.count
-
-    def __hash__(self):
-        return hash(self.name)
+    @property
+    def key(self):
+        return self.count
 
 
 class UnicodeCharSeq(Type):
@@ -397,34 +506,26 @@ class UnicodeCharSeq(Type):
         name = "[unichr x %d]" % count
         super(UnicodeCharSeq, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, UnicodeCharSeq):
-            return self.count == other.count
-
-    def __hash__(self):
-        return hash(self.name)
+    @property
+    def key(self):
+        return self.count
 
 
 class Record(Type):
     mutable = True
 
-    def __init__(self, id, fields, size, align, dtype):
+    def __init__(self, id, fields, size, aligned, dtype):
         self.id = id
         self.fields = fields.copy()
         self.size = size
-        self.align = align
+        self.aligned = aligned
         self.dtype = dtype
         name = 'Record(%s)' % id
         super(Record, self).__init__(name)
 
-    def __eq__(self, other):
-        if isinstance(other, Record):
-            return (self.id == other.id and
-                    self.size == other.size and
-                    self.align == other.align)
-
-    def __hash__(self):
-        return hash(self.name)
+    @property
+    def key(self):
+        return (self.dtype, self.size, self.aligned)
 
     def __len__(self):
         return len(self.fields)
@@ -445,7 +546,8 @@ class ArrayIterator(IteratorType):
     def __init__(self, array_type):
         self.array_type = array_type
         name = "iter(%s)" % (self.array_type,)
-        if array_type.ndim == 1:
+        nd = array_type.ndim
+        if nd == 0 or nd == 1:
             self.yield_type = array_type.dtype
         else:
             self.yield_type = array_type.copy(ndim=array_type.ndim - 1)
@@ -453,16 +555,13 @@ class ArrayIterator(IteratorType):
 
 
 class Array(IterableType):
-    __slots__ = 'dtype', 'ndim', 'layout'
 
     mutable = True
 
     # CS and FS are not reserved for inner contig but strided
     LAYOUTS = frozenset(['C', 'F', 'CS', 'FS', 'A'])
 
-    def __init__(self, dtype, ndim, layout):
-        from numba.typeconv.rules import default_type_manager as tm
-
+    def __init__(self, dtype, ndim, layout, const=False):
         if isinstance(dtype, Array):
             raise TypeError("Array dtype cannot be Array")
         if layout not in self.LAYOUTS:
@@ -471,23 +570,33 @@ class Array(IterableType):
         self.dtype = dtype
         self.ndim = ndim
         self.layout = layout
-        name = "array(%s, %sd, %s)" % (dtype, ndim, layout)
+        self.const = const
+        name = "array(%s, %sd, %s, %s)" % (dtype, ndim, layout,
+                                           {True: 'const',
+                                            False: 'nonconst'}[const])
         super(Array, self).__init__(name, param=True)
         self.iterator_type = ArrayIterator(self)
 
-        if layout != 'A':
-            # Install conversion from non-any layout to any layout
-            ary_any = Array(dtype, ndim, 'A')
+    def post_init(self):
+        """
+        Install conversion from this layout (if non-'A') to 'A' layout.
+        """
+        if self.layout != 'A':
+            from numba.typeconv.rules import default_type_manager as tm
+            ary_any = Array(self.dtype, self.ndim, 'A', const=self.const)
+            # XXX This will make the types immortal
             tm.set_safe_convert(self, ary_any)
 
-    def copy(self, dtype=None, ndim=None, layout=None):
+    def copy(self, dtype=None, ndim=None, layout=None, const=None):
         if dtype is None:
             dtype = self.dtype
         if ndim is None:
             ndim = self.ndim
         if layout is None:
             layout = self.layout
-        return Array(dtype=dtype, ndim=ndim, layout=layout)
+        if const is None:
+            const = self.const
+        return Array(dtype=dtype, ndim=ndim, layout=layout, const=const)
 
     def get_layout(self, dim):
         assert 0 <= dim < self.ndim
@@ -515,15 +624,9 @@ class Array(IterableType):
         """
         return intp, self.dtype
 
-    def __eq__(self, other):
-        if isinstance(other, Array):
-            return (self.dtype == other.dtype and
-                    self.ndim == other.ndim and
-                    self.layout == other.layout)
-
-    def __hash__(self):
-        return hash((self.dtype, self.ndim, self.layout))
-
+    @property
+    def key(self):
+        return self.dtype, self.ndim, self.layout, self.const
 
     @property
     def is_c_contig(self):
@@ -566,12 +669,9 @@ class UniTuple(IterableType):
     def __len__(self):
         return self.count
 
-    def __eq__(self, other):
-        if isinstance(other, UniTuple):
-            return self.dtype == other.dtype and self.count == other.count
-
-    def __hash__(self):
-        return hash((self.dtype, self.count))
+    @property
+    def key(self):
+        return self.dtype, self.count
 
 
 class UniTupleIter(IteratorType):
@@ -582,12 +682,9 @@ class UniTupleIter(IteratorType):
         name = 'iter(%s)' % unituple
         super(UniTupleIter, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, UniTupleIter):
-            return self.unituple == other.unituple
-
-    def __hash__(self):
-        return hash(self.unituple)
+    @property
+    def key(self):
+        return self.unituple
 
 
 class Tuple(Type):
@@ -606,12 +703,9 @@ class Tuple(Type):
     def __len__(self):
         return len(self.types)
 
-    def __eq__(self, other):
-        if isinstance(other, Tuple):
-            return self.types == other.types
-
-    def __hash__(self):
-        return hash(self.types)
+    @property
+    def key(self):
+        return self.types
 
     def __iter__(self):
         return iter(self.types)
@@ -625,12 +719,9 @@ class CPointer(Type):
         name = "*%s" % dtype
         super(CPointer, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, CPointer):
-            return self.dtype == other.dtype
-
-    def __hash__(self):
-        return hash(self.dtype)
+    @property
+    def key(self):
+        return self.dtype
 
 
 class Object(Type):
@@ -641,26 +732,52 @@ class Object(Type):
         name = "Object(%s)" % clsobj.__name__
         super(Object, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, Object):
-            return self.cls == other.cls
-
-    def __hash__(self):
-        return hash(self.cls)
+    @property
+    def key(self):
+        return self.cls
 
 
 class Optional(Type):
     def __init__(self, typ):
+        assert typ != none
+        assert not isinstance(typ, Optional)
         self.type = typ
         name = "?%s" % typ
         super(Optional, self).__init__(name, param=True)
 
-    def __eq__(self, other):
-        if isinstance(other, Optional):
-            return self.type == other.type
+    def post_init(self):
+        """
+        Install conversion from optional(T) to T
+        """
+        from numba.typeconv.rules import default_type_manager as tm
 
-    def __hash__(self):
-        return hash(self.type)
+        # TODO make type manager remember all cast relation
+        #      so that new rule will propagate
+        tm.set_safe_convert(self, self.type)
+        tm.set_promote(self.type, self)
+        tm.set_promote(none, self)
+
+        if self.type in number_domain:
+            for t in number_domain - set([self.type]):
+                tcc = tm.check_compatible(t, self.type)
+                if tcc == 'promote':
+                    tm.set_promote(t, self)
+                elif tcc == 'safe':
+                    tm.set_safe_convert(t, self)
+                elif tcc == 'unsafe':
+                    tm.set_unsafe_convert(t, self)
+                else:
+                    assert tcc is None, tcc
+
+        if isinstance(self.type, Array):
+            if self.type.layout == 'A':
+                tm.set_promote(self.type.copy(layout='C'), self)
+                tm.set_promote(self.type.copy(layout='F'), self)
+
+
+    @property
+    def key(self):
+        return self.type
 
 
 # Utils
@@ -676,15 +793,15 @@ def is_int_tuple(x):
 # Short names
 
 
-pyobject = Type('pyobject')
-none = Dummy('none')
-Any = Dummy('any')
-VarArg = Dummy('...')
-string = Dummy('str')
+pyobject = Opaque('pyobject')
+none = Opaque('none')
+Any = Phantom('any')
+VarArg = Phantom('...')
+string = Opaque('str')
 
 # No operation is defined on voidptr
 # Can only pass it around
-voidptr = Dummy('void*')
+voidptr = Opaque('void*')
 
 boolean = bool_ = Type('bool')
 
@@ -706,15 +823,15 @@ float64 = Float('float64')
 complex64 = Complex('complex64', float32)
 complex128 = Complex('complex128', float64)
 
-len_type = Dummy('len')
-range_type = Dummy('range')
-slice_type = Dummy('slice')
-abs_type = Dummy('abs')
-neg_type = Dummy('neg')
-print_type = Dummy('print')
-print_item_type = Dummy('print-item')
-sign_type = Dummy('sign')
-exception_type = Dummy('exception')
+len_type = Phantom('len')
+range_type = Phantom('range')
+slice_type = Phantom('slice')
+abs_type = Phantom('abs')
+neg_type = Phantom('neg')
+print_type = Phantom('print')
+print_item_type = Phantom('print-item')
+sign_type = Phantom('sign')
+exception_type = Phantom('exception')
 
 range_iter32_type = RangeIteratorType('range_iter32', int32)
 range_iter64_type = RangeIteratorType('range_iter64', int64)
@@ -769,6 +886,9 @@ ulong = _make_unsigned(numpy.long)
 longlong = _make_signed(numpy.longlong)
 ulonglong = _make_unsigned(numpy.longlong)
 
+# optional types
+optional = Optional
+
 __all__ = '''
 int8
 int16
@@ -814,4 +934,5 @@ f4
 f8
 c8
 c16
+optional
 '''.split()
